package authentication

import (
	"bytes"
	"context"
	"net/http"

	"git.codemonkeysoftware.net/mouseion/server/pkg/views"

	"github.com/volatiletech/authboss/v3"
)

// TemplateSetRenderer adapts a TemplateSet as an authboss.Renderer.
type TemplateSetRenderer struct {
	templateSet views.TemplateSet
	pathPrefix  string
}

func NewTemplateSetRenderer(templateSet views.TemplateSet, pathPrefix string) *TemplateSetRenderer {
	return &TemplateSetRenderer{
		templateSet: templateSet,
		pathPrefix:  pathPrefix,
	}
}

func (renderer *TemplateSetRenderer) Load(names ...string) error {
	return nil
}

func (renderer *TemplateSetRenderer) Render(ctx context.Context, path string, data authboss.HTMLData) (output []byte, contentType string, err error) {
	buffer := bytes.NewBuffer(nil)
	err = renderer.templateSet.Execute(buffer, renderer.pathPrefix+path, nil, data)
	if err != nil {
		return nil, "", err
	}

	return buffer.Bytes(), "text/html", nil
}

// HybridRenderer will use JSON if the request was a JSON request, otherwise will use the TemplateSetRenderer
type HybridRenderer struct {
	templateRenderer authboss.Renderer
	jsonRenderer     authboss.Renderer
}

func NewHybridRenderer(tempalateRenderer authboss.Renderer, jsonRenderer authboss.Renderer) authboss.Renderer {
	return &HybridRenderer{
		templateRenderer: tempalateRenderer,
		jsonRenderer:     jsonRenderer,
	}
}
func (renderer *HybridRenderer) Load(names ...string) error {
	return nil
}

func (renderer *HybridRenderer) Render(ctx context.Context, path string, data authboss.HTMLData) (output []byte, contentType string, err error) {
	if ctx.Value(contentTypeKey) == contentTypeJSON {
		return renderer.jsonRenderer.Render(ctx, path, data)
	}
	return renderer.templateRenderer.Render(ctx, path, data)

}

type contextKey string

const (
	contentTypeKey contextKey = "Content-Type"

	contentTypeJSON = "application/json"
)

func SetRequestType(next http.Handler) http.Handler {
	f := func(w http.ResponseWriter, req *http.Request) {
		contentType := req.Header.Get("Content-Type")

		ctx := context.WithValue(req.Context(), contentTypeKey, contentType)
		req = req.WithContext(ctx)

		next.ServeHTTP(w, req)
	}
	return http.HandlerFunc(f)
}
