package authentication

import (
	"errors"

	"github.com/volatiletech/authboss/v3"
)

var ErrWrongType = errors.New("user is not an AuthUser")

func GetAuthUser(user authboss.User) (*User, error) {
	authUser, ok := user.(*User)
	if !ok {
		return nil, ErrWrongType
	}
	return authUser, nil
}

type User struct {
	ID       int
	Email    string
	Password string

	ConfirmSelector string `db:"confirm_selector"`
	ConfirmVerifier string `db:"confirm_verifier"`
	Confirmed       bool
}

func (u *User) GetPID() (email string)         { return u.Email }
func (u *User) PutPID(email string)            { u.Email = email }
func (u *User) GetPassword() (password string) { return u.Password }
func (u *User) PutPassword(password string)    { u.Password = password }

func (u *User) GetEmail() (email string)              { return u.Email }
func (u *User) PutEmail(email string)                 { u.Email = email }
func (u *User) GetConfirmed() (confirmed bool)        { return u.Confirmed }
func (u *User) PutConfirmed(confirmed bool)           { u.Confirmed = confirmed }
func (u *User) GetConfirmSelector() (selector string) { return u.ConfirmSelector }
func (u *User) PutConfirmSelector(selector string)    { u.ConfirmSelector = selector }
func (u *User) GetConfirmVerifier() (verifier string) { return u.ConfirmVerifier }
func (u *User) PutConfirmVerifier(verifier string)    { u.ConfirmVerifier = verifier }
