package email_test

import (
	"errors"
	"testing"

	"git.codemonkeysoftware.net/mouseion/server/pkg/email"

	"gopkg.in/gomail.v2"
)

func TestSetMultiAddressHeader(t *testing.T) {
	addresses := []string{
		"a@b.c",
		"e@f.g",
		"h@i.j",
		"k@l.m",
		"n@o.p",
	}
	names := []string{"a", "b", "c", "d", "e"}
	const headerName = "To"

	checkHeaderLength := func(msg *gomail.Message) {
		t.Helper()
		header := msg.GetHeader(headerName)
		if len(header) != len(addresses) {
			t.Errorf("header is wrong length: expected %d, got %d", len(addresses), len(header))
		}
	}

	t.Run("succeeds if names is empty", func(t *testing.T) {
		msg := gomail.NewMessage()
		err := email.SetMultiAddressHeader(msg, headerName, addresses, nil)
		if err != nil {
			t.Errorf("expected no error: %v", err)
		}
		checkHeaderLength(msg)
	})

	t.Run("succeeds if names and addresses have same length", func(t *testing.T) {
		msg := gomail.NewMessage()
		err := email.SetMultiAddressHeader(msg, headerName, addresses, names)
		if err != nil {
			t.Errorf("expected no error: %v", err)
		}
		checkHeaderLength(msg)
	})

	t.Run("fails if names and addresses have different lengths", func(t *testing.T) {
		msg := gomail.NewMessage()
		err := email.SetMultiAddressHeader(msg, headerName, addresses, names[1:])
		var expectedErr = new(email.AddressNameMismatchError)
		if !errors.As(err, &expectedErr) {
			t.Errorf("expected AddressNameMismatchError, got %q", err)
		}
	})
}
