package entry_test

import (
	"testing"

	"git.codemonkeysoftware.net/mouseion/server/pkg/entry"
)

func expectStringsEqual(t *testing.T, actual, expected string) {
	t.Helper()
	if actual != expected {
		t.Logf("expected %q, got %q", expected, actual)
		t.Fail()
	}
}

func TestTruncateToCodePoint(t *testing.T) {
	t.Run("does nothing to empty string", func(t *testing.T) {
		result := entry.TruncateToCodePoint(10, "")
		expectStringsEqual(t, result, "")
	})

	t.Run("does nothing to string of maxLength", func(t *testing.T) {
		const s = "αβψδεφ"
		result := entry.TruncateToCodePoint(len(s), s)
		expectStringsEqual(t, result, s)
	})

	t.Run("truncates at a code point boundary", func(t *testing.T) {
		const prefix = "αβψδεφ"
		for _, suffix := range []string{"A", "ç", "→", "🍆"} {
			combined := prefix + suffix
			for maxLength := len(prefix); maxLength < len(combined); maxLength++ {
				result := entry.TruncateToCodePoint(maxLength, combined)
				expectStringsEqual(t, result, prefix)
			}
		}
	})

	t.Run("truncates string at maxLength if it ends with an invalid byte sequence", func(t *testing.T) {
		const prefix = "abc"
		const invalidSuffix = "\xf0\x9f\x8d"
		const combined = prefix + invalidSuffix
		for maxLength := len(prefix); maxLength <= len(combined); maxLength++ {
			result := entry.TruncateToCodePoint(maxLength, combined)
			expectStringsEqual(t, result, combined[:maxLength])
		}
	})
}
