package webserver

import (
	"encoding/json"
	"fmt"
	"log"
	"net/http"

	"git.codemonkeysoftware.net/mouseion/server/pkg/authentication/confirm"
	"github.com/go-chi/chi/v5"
	"github.com/volatiletech/authboss/v3"
)

func (server *Server) accountRoutes(mux chi.Router) {
	mux.Get("/confirmation_pending", server.staticTemplate("pages/account/confirmation_pending"))

	confirmed := mux.With(server.requireLoggedIn, confirm.Middleware(server.ab))
	confirmed.Post("/change_email", server.changeEmail)

}

type changeEmailCommand struct {
	Email string `json:"email"`
}

func (server *Server) changeEmail(w http.ResponseWriter, r *http.Request) {
	defer r.Body.Close()
	decoder := json.NewDecoder(r.Body)
	cmd := changeEmailCommand{}
	err := decoder.Decode(&cmd)
	if err != nil {
		log.Println(err)
		fmt.Fprint(w, err)
		return
	}

	user, err := server.ab.CurrentUser(r)
	if err != nil {
		log.Println(err)
		fmt.Fprintln(w, err)
		return
	}
	oldEmail := user.GetPID()
	user.PutPID(cmd.Email)
	err = server.authStorer.Save(r.Context(), user)
	if err != nil {
		log.Println(err)
		fmt.Fprint(w, err)
		return
	}

	log.Printf("user %s changed email to %s", oldEmail, cmd.Email)
	authboss.PutSession(w, authboss.SessionKey, cmd.Email)
	w.WriteHeader(http.StatusOK)
}

func (server *Server) staticTemplate(templateName string) http.HandlerFunc {
	return func(rw http.ResponseWriter, r *http.Request) {
		err := server.templates.Execute(rw, templateName, nil, nil)
		if err != nil {
			log.Println(err)
		}
	}
}
