package webserver

import (
	"fmt"
	"log"
	"net/http"

	"git.codemonkeysoftware.net/mouseion/server/pkg/views"
	"github.com/go-chi/chi/v5"
	"github.com/volatiletech/authboss/v3"
)

func (server *Server) webRoutes(mux chi.Router) {
	mux.Get("/dashboard", server.dashboard)
	mux.Get("/search", server.search)
}

func (server *Server) dashboard(w http.ResponseWriter, r *http.Request) {
	user, err := server.ab.CurrentUser(r)
	if err != nil {
		log.Println(err)
		fmt.Fprintln(w, err)
		return
	}

	email := user.GetPID()
	tokens, err := server.tokenService.ListTokens(r.Context(), email)
	if err != nil {
		log.Println(err)
		fmt.Fprintln(w, err)
		return
	}
	data := authboss.HTMLData{
		"tokens": tokens,
	}
	err = server.templates.Execute(w, "/pages/dashboard", nil, data)
	if err != nil {
		log.Println(err)
		return
	}
}

func (server *Server) search(w http.ResponseWriter, r *http.Request) {
	entries, err := server.searchEntries(r)
	if err != nil {
		log.Println(err)
		w.WriteHeader(http.StatusInternalServerError)
		fmt.Fprint(w, err)
		return
	}
	data := views.Map{"entries": entries}
	err = server.templates.Execute(w, "pages/search_results", nil, data)
	if err != nil {
		log.Println(err)
	}
}
