package webserver

import (
	"io/fs"
	"log"
	"net/http"
	"strings"

	"github.com/go-chi/chi/v5"
)

func (server *Server) mountStatic(mux *chi.Mux) {
	err := mountFS(mux, server.public, "public/css", "/css")
	if err != nil {
		log.Fatal(err)
	}

	err = mountFS(mux, server.public, "public/js", "/js")
	if err != nil {
		log.Fatal(err)
	}
	err = mountFS(mux, server.public, "public/img", "/img")
	if err != nil {
		log.Fatal(err)
	}
}

func mountFS(mux *chi.Mux, dir fs.FS, subdir string, mountPath string) error {
	subDir, err := fs.Sub(dir, subdir)
	if err != nil {
		return err
	}
	mux.Get(mountPath+"/*", func(w http.ResponseWriter, r *http.Request) {
		rctx := chi.RouteContext(r.Context())
		pathPrefix := strings.TrimSuffix(rctx.RoutePattern(), "/*")
		fs := http.StripPrefix(pathPrefix, http.FileServer(http.FS(subDir)))
		fs.ServeHTTP(w, r)
	})
	return nil
}
